@@ -6,9 +6,8 @@ Builds the MANIFEST file for use in building distributions
 Normal sdist calls do not include the docs directory, this code corrects
 that.
 
-project home: http://code.google.com/p/wyckedsceptre/
-Last Changed Date: $LastChangedDate:                                         $
-Revision         : $Rev:  $
+$LastChangedDate:                                                $
+Revision         : $Rev:         $
 """
 
 
@@ -27,7 +26,10 @@ def main():
         for root, _, files in os.walk(root_path):
             for filename in fnmatch.filter(files, pattern):
                 filepath = os.path.join(root, filename)
-                if filepath.find('svn') != -1:
+                if filepath.find('svn') != -1 or \
+                        filepath.find('dist/') != -1 or \
+                        filepath.find('.pyc') != -1 or \
+                        filepath.find('.py~') != -1:
                     continue
                 else:
                     filepath = filepath[2:] + '\n'
