@@ -2,8 +2,9 @@
 # test_release.sh
 
 # run the python main files
-python wyckedsceptre/wyckedsceptre.py
 python test/test_wyckedsceptre.py
+python examples/encounter_example.py
+python examples/example.py
 
 # check all Python files for pep8 compatability
 pep8 --repeat --ignore=errors --verbose *.py
@@ -21,5 +22,6 @@ pylint wyckedsceptre/*.py
 pylint test/*.py
 
 python -3 test/test_wyckedsceptre.py
-python -3 wyckedsceptre/wyckedsceptre.py
+python -3 examples/encounter_example.py
+python -3 examples/example.py
 
