@@ -12,8 +12,8 @@ and documentation.
 @todo Make public members private
 
 project home: http://code.google.com/p/wyckedsceptre/
-Last Changed Date: $LastChangedDate: 2010-10-26 22:37:34 -0500 (Tue, 26 Oct 2$
-Revision         : $Rev: 50 $
+$LastChangedDate: 2010-11-05 23:45:39 -0500 (Fri, 05 Nov 2010) $
+$Rev: 66 $
 """
 import wyckedsceptre_constants
 
@@ -33,10 +33,6 @@ def assert_positive_integer(val_to_test, val_name='Value'):
 class Die(object):
     """
     Object for simulating a set of an m-numbered set of n-sided dice
-
-    >>> print wyckedsceptre.Die('1d20')
-    1d20
-
     """
 
     def __init__(self, die_str='1d20'):
@@ -71,6 +67,9 @@ class Die(object):
         Throws a ValueError exception if any problems are detected.
         @exception ValueError if # of sides or dice is invalid
         """
+        if not wyckedsceptre_constants.DEBUG:
+            return
+
         if not (self.die_sides in wyckedsceptre_constants.VALID_DIE_SIDES):
             raise ValueError('Number of die sides must be either ' + \
                     str(wyckedsceptre_constants.VALID_DIE_SIDES))
@@ -145,6 +144,14 @@ class Attack(object):
         else:
             return False
 
+    def __str__(self):
+        """
+        String conversion routine to return ex: 1d6 + 4
+        @todo simplify to no + if bonus is zero
+        """
+        return str(self._damage_die) + ' + ' + str(self._bonus) + ' ' + \
+                self._desc
+
     def highest(self):
         """
         The highest possible outcome of an attack roll
@@ -259,10 +266,12 @@ class Character(object):
         @exception ValueError a property is not within a valid range
         @exception TypeError a property is not the right data type
         """
+        if not wyckedsceptre_constants.DEBUG:  # added for speed in profiling
+            return
         assert_positive_integer(self.xp, 'Character - XP')
         assert_positive_integer(self.armor_class, 'Character - Armor class')
         assert_positive_integer(self.money, 'Character - Money')
-        assert_positive_integer(self.initiative_bonus, 'Initiative bonus')
+        # assert_positive_integer(self.initiative_bonus, 'Initiative bonus')
         assert_positive_integer(self.initiative_total, 'Initiative total')
         assert_positive_integer(self.hit_points_max, 'Char - max hit points')
         assert_positive_integer(self.level, 'Character - level')
@@ -282,7 +291,8 @@ class Character(object):
         """
         res = self.name + ' the level ' + str(self.level) + ' ' + \
                 self.dnd_class + ' HP: ' + str(self.hit_points) + '/' + \
-                str(self.hit_points_max)
+                str(self.hit_points_max) + ' Init: ' + \
+                str(self.initiative_total)
         return res
 
     def get_hp(self):
@@ -348,8 +358,8 @@ class Character(object):
         Computes if the character is currently concious (hit_points > 0)
         @returns True if the character's hit_points is greater than zero.
         """
-        self.update_state()
-        return self.state > 0
+        self.update_state()  ## cut for speed in profiling
+        return (self.state > 0)
 
     def update_state(self):
         """
@@ -362,7 +372,7 @@ class Character(object):
         self.__invariant__()
         # force true division
         self.state = self.get_hp() / float(self.hit_points_max)
-        self._level_up_()
+        # self._level_up_()  ## cut for speed in profiling
         self.__invariant__()
         return self.state
 
@@ -459,6 +469,22 @@ class Encounter(object):
         ## An iterator pointing to the current player's turn
         self.active_turn = iter(self.run_order)
 
+    def encounter_statistics(self, num_rounds=100):
+        """
+        runs the encounter num_rounds and returns percentage of the time
+        that the Players won.
+        @todo reset the player stats after a round
+        """
+        if num_rounds < 1:
+            return 0
+        rounds_players_won = 0
+
+        for _ in range(num_rounds):
+            self.reset_all_health()
+            if self.run_encounter():
+                rounds_players_won += 1
+        return rounds_players_won / float(num_rounds)
+
     def run_encounter(self):
         """
         Start simulating an encounter after it has been properly initialized
@@ -466,11 +492,33 @@ class Encounter(object):
         parties are unconcious or dead
         @todo implement run_encounter
         """
+
+        self.run_order = []
         # roll initiatives
+        for iter_char in self.players:
+            iter_char.roll_initiative()
+            self.run_order.append(iter_char)
+        for iter_char in self.npcs:
+            iter_char.roll_initiative()
+            self.run_order.append(iter_char)
+
+        # sort the characters by initative
+        self.run_order.sort(key=lambda x: x.initiative_total, reverse=True)
+#        print '\nPRE-FIGHT'
+#        for iter_char in self.run_order:
+#            print iter_char.__str__()
+
         # while both parties have concious people
-        #  __run_round__
-        # return results
-        pass
+        num_rounds = 0
+        while self.players_any_concious() and self.npcs_any_concious():
+            # do 1 round of turns
+            self.__run_round__()
+            num_rounds += 1
+
+#        print '\nPOST FIGHT CHARACTER STATES - TOOK ' + str(num_rounds) + ' rounds'
+#        for iter_char in self.run_order:
+#            print iter_char.__str__()
+        return self.players_any_concious()
 
     def __run_round__(self):
         """
@@ -481,12 +529,54 @@ class Encounter(object):
                  the characters have been ordered accordingly
         @todo implement __run_round__
         """
+        import random
+        num_chars = len(self.run_order)
+
         # iterate thru each character's turn
+        for focused_char in self.run_order:
+            # check to see if one of the teams is dead/unconcious, end if so
+            if self.players_any_concious() and self.npcs_any_concious():
+                if focused_char.is_concious():  # is the current player alive?
+
+                    # pick an enemy target at random
+                    target_index = -1
+                    while target_index == -1:  # @todo possible infinite loop
+                        target_index = random.randint(0, num_chars - 1)
+
+                        if self.run_order[target_index].is_npc == focused_char.is_npc:
+                            # are they on the same team?
+#                            print focused_char.name + ' is on same team as ' + self.run_order[target_index].name
+                            target_index = -1
+                        elif not self.run_order[target_index].is_concious():
+                            # target isn't concious
+                            target_index = -1
+                        else:
+                            # a valid target
+                            break
+
+                    # a valid target has been found
+ #                   print focused_char.name + ' attacks ' + self.run_order[target_index].name
+                    [hit_roll, damage] = focused_char.make_attack()
+                    if hit_roll >= self.run_order[target_index].get_ac():
+#                        print focused_char.name + ' attack on ' + self.run_order[target_index].name + ' HIT (' + \
+#                            str(hit_roll) + 'v' + str(self.run_order[target_index].get_ac()) + ') & did ' + \
+#                            str(damage) + ' damage'
+                        self.run_order[target_index].take_damage(damage)
+#                    else:
+#                        print focused_char.name + ' attack on ' + self.run_order[target_index].name + ' MISSED (' + \
+#                            str(hit_roll) + 'v' + str(self.run_order[target_index].get_ac()) + ')'
+#                    print self.run_order[target_index]
+                else:
+                    continue  # this player does not get a turn
+            else:   # this fight is over
+                return self.players_any_concious()
+        return self.players_any_concious()
+
+
         #    check if round or continuing is necessary
         #    make an attack on another character
         #    apply damage to target
         # update each player's status
-        pass
 
     def add_character(self, new_char, group):
         """
@@ -499,11 +589,12 @@ class Encounter(object):
         @todo add return values
         """
         if 'players' == group:
-            self.players.insert(new_char)
+            self.players.append(new_char)
         elif 'npcs' == group:
-            self.npcs.insert(new_char)
-#        else:
-#            raise Encounter.GroupNotSpecified
+            self.npcs.append(new_char)
+        else:
+            raise ValueError('group must be players or npcs')
+        return True
 
     def players_any_concious(self):
         """
@@ -511,12 +602,20 @@ class Encounter(object):
         @returns True if at least 1 member of the players group is concious
         @returns False if there are no members in the players group or if none
         are concious
+        @todo make it reference the current values
         """
-        for member in self.players:
-            if member.is_concious():
+        for member in self.run_order:
+            if (not member.is_npc) and member.is_concious():
                 return True
         return False
 
+    def reset_all_health(self):
+        """
+        restores all characters to max hit points
+        """
+        for iter_char in self.run_order:
+            iter_char.hit_points = iter_char.hit_points_max
+
     def npcs_any_concious(self):
         """
         Determines if any NPC characters are currently concious
@@ -524,22 +623,11 @@ class Encounter(object):
         @returns False if there are no members in the npcs group or if none
         are concious
         """
-        for member in self.npcs:
-            if member.is_concious():
+        for member in self.run_order:
+            if member.is_npc and member.is_concious():
                 return True
         return False
 
-    def roll_initiatives(self):
-        """
-        Rolls initiatives for each character in the encounter and sorts them
-        according to initiative.
-        @todo need pointers to character objs
-        """
-        init_rolls = []
-        for character in self.run_order:
-            init_rolls.append(character.roll_initiative())
-        self.run_order.sort(key=lambda x: x.initiative_total, reverse=True)
-
 
 def load_file(filename):
     """
